from typing import Optional, List

from magic_eden.api.utils.consts import METokenResponse, MEActivityResponse, MEOfferResponse, \
    MEEscrowBalanceResponse, MEListingResponse, MECollectionStatsResponse, MELaunchpadCollectionResponse, \
    MECollectionResponse
from magic_eden.api.utils.types import METoken, MEActivity, MEOffer, MEEscrowBalance, MEListingItem, \
    MECollectionStats, MECollection
from .converters import convert_sol_absolute
from .datetime import parse_timestamp, parse_date


def list_map(func, collection: Optional[List]):
    return None if collection is None else list(map(func, collection))


def token_response_mapper(data: dict) -> METoken:
    return {
        'mint_address': data.get(METokenResponse.mint_address, ''),
        'owner': data.get(METokenResponse.owner, ''),
        'supply': data.get(METokenResponse.supply, 0),
        'delegate': data.get(METokenResponse.delegate, ''),
        'symbol': data.get(METokenResponse.symbol, ''),
        'name': data.get(METokenResponse.name, ''),
        'update_authority': data.get(METokenResponse.update_authority, ''),
        'primary_sale_happened': data.get(METokenResponse.primary_sale_happened, 0),
        'seller_fee_basis_points': data.get(METokenResponse.seller_fee_bases_point, 0),
        'image': data.get(METokenResponse.image, ''),
        'animation_url': data.get(METokenResponse.animation_url, None),
        'external_url': data.get(METokenResponse.external_url, ''),
        'attributes': data.get(METokenResponse.attributes, []),
        'properties': data.get(METokenResponse.properties, {})
    }


_ACTIVITY_TYPE_MAPPING = {
    'bid': 'BID',
    'list': 'LIST',
    'buyNow': 'BUY_NOW',
    'cancelBid': 'CANCEL_BID',
    'delist': 'DELIST',
}


def activity_response_mapper(data: dict) -> MEActivity:
    _type = data.get(MEActivityResponse.type, None)
    if _type:
        _type = _ACTIVITY_TYPE_MAPPING.get(_type, None)
    return {
        'signature': data.get(MEActivityResponse.signature, ''),
        'type': _type,
        'source': data.get(MEActivityResponse.source, ''),
        'token_mint': data.get(MEActivityResponse.token_mint, ''),
        'symbol': data.get(MEActivityResponse.symbol, ''),
        'slot': data.get(MEActivityResponse.slot, ''),
        'block_time': parse_timestamp(data.get(MEActivityResponse.block_time, 0)),
        'buyer': data.get(MEActivityResponse.buyer, ''),
        'buyer_referral': data.get(MEActivityResponse.buyer_referral, ''),
        'seller': data.get(MEActivityResponse.seller, ''),
        'seller_referal': data.get(MEActivityResponse.seller_referral, ''),
        'price': data.get(MEActivityResponse.price, 0),
    }


def offer_response_mapper(data: dict) -> MEOffer:
    return {
        'pda_address': data.get(MEOfferResponse.pda_address, ''),
        'token_mint': data.get(MEOfferResponse.token_mint, ''),
        'auction_house': data.get(MEOfferResponse.auction_house, ''),
        'buyer': data.get(MEOfferResponse.buyer, ''),
        'price': data.get(MEOfferResponse.price, 0),
        'token_size': data.get(MEOfferResponse.token_size, 0),
        'expiry': data.get(MEOfferResponse.expiry, 0)
    }


def escrow_balance_response_mapper(data: dict) -> MEEscrowBalance:
    return {
        'buyer_escrow': data.get(MEEscrowBalanceResponse.buyer_escrow, ''),
        'balance': data.get(MEEscrowBalanceResponse.balance, 0)
    }


def collections_response_mapper(data: dict) -> MECollection:
    return {
        'symbol': data.get(MECollectionResponse.symbol, ''),
        'name': data.get(MECollectionResponse.name, ''),
        'description': data.get(MECollectionResponse.description, ''),
        'image': data.get(MECollectionResponse.image, ''),
        'twitter': data.get(MECollectionResponse.twitter, ''),
        'discord': data.get(MECollectionResponse.discord, ''),
        'website': data.get(MECollectionResponse.website, ''),
        'categories': data.get(MECollectionResponse.categories, []),
        'is_flagged': data.get(MECollectionResponse.is_flagged, None)
    }


def listing_response_mapper(data: dict) -> MEListingItem:
    return {
        'pda_address': data.get(MEListingResponse.pda_address, ''),
        'auction_house': data.get(MEListingResponse.auction_house, ''),
        'token_address': data.get(MEListingResponse.token_address, ''),
        'token_mint': data.get(MEListingResponse.token_mint, ''),
        'seller': data.get(MEListingResponse.seller, ''),
        'token_size': data.get(MEListingResponse.token_size, 0),
        'price': data.get(MEListingResponse.price, 0)
    }


def collection_stats_mapper(data: dict) -> MECollectionStats:
    return {
        'symbol': data.get(MECollectionStatsResponse.symbol, ''),
        'floor_price': convert_sol_absolute(data.get(MECollectionStatsResponse.floor_price, 0)),
        'listed_count': data.get(MECollectionStatsResponse.listed_count, 0),
        'volume_all': convert_sol_absolute(data.get(MECollectionStatsResponse.volume_all, 0))
    }


def launchpad_collection_mapper(data: dict):
    return {
        'symbol': data.get(MELaunchpadCollectionResponse.symbol, ''),
        'name': data.get(MELaunchpadCollectionResponse.name, ''),
        'description': data.get(MELaunchpadCollectionResponse.description, ''),
        'featured': data.get(MELaunchpadCollectionResponse.featured, None),
        'edition': data.get(MELaunchpadCollectionResponse.edition, ''),
        'image': data.get(MELaunchpadCollectionResponse.image, ''),
        'price': data.get(MELaunchpadCollectionResponse.price, 0),
        'supply': data.get(MELaunchpadCollectionResponse.supply, 0),
        'launch_datetime': parse_date(data.get(MELaunchpadCollectionResponse.launch_datetime))
    }



from magic_eden.api.utils.types import MagicEdenAPIEnvironment


class MEUrlBuilder:
    _endpoint: str

    def __init__(self, environment: MagicEdenAPIEnvironment = 'MAINNET'):
        if environment == 'DEVNET':
            self._endpoint = 'https://api-devnet.magiceden.dev/v2'
        elif environment == 'MAINNET':
            self._endpoint = 'https://api-mainnet.magiceden.dev/v2'
        else:
            raise ValueError(f'Invalid environment expected {MagicEdenAPIEnvironment} but get {environment}')


class MEAPITokensUrlBuilder(MEUrlBuilder):
    def get_token(self, token: str):
        return f'{self._endpoint}/tokens/{token}'

    def listings(self, token):
        return f'{self._endpoint}/tokens/{token}/listings'

    def offer_received(self, token, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/tokens/{token}/offer_received?offset={offset}&limit={limit}'

    def activities(self, token, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/tokens/{token}/activities?offset={offset}&limit={limit}'


class MEAPIWalletUrlsBuilder(MEUrlBuilder):
    def tokens(self, wallet_address: str, offset: int = 0, limit: int = 100, listed_only: bool = None):
        res = f'{self._endpoint}/wallets/{wallet_address}/tokens' \
              f'?offset={offset}&limit={limit}'
        if listed_only is not None:
            res = f'{res}&listedOnly={str(listed_only).lower()}'
        return res

    def activities(self, wallet_address: str, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/wallets/{wallet_address}/activities?offset={offset}&limit={limit}'

    def offers_made(self, wallet_address: str, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/wallets/{wallet_address}/activities?offset={offset}&limit={limit}'

    def offers_received(self, wallet_address: str, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/wallets/{wallet_address}/offers_received?offset={offset}&limit={limit}'

    def escrow_balance(self, wallet_address: str):
        return f'{self._endpoint}/wallets/{wallet_address}/escrow_balance'


class MEAPICollectionsUrlsBuilder(MEUrlBuilder):
    def collections(self, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/collections?offset={offset}&limit={limit}'

    def listings(self, symbol: str, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/collections/{symbol}/listings?offset={offset}&limit={limit}'

    def activities(self, symbol: str, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/collections/{symbol}/activities?offset={offset}&limit={limit}'

    def stats(self, symbol: str, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/collections/{symbol}/stats?offset={offset}&limit={limit}'


class MEAPILaunchpadUrlsBuilder(MEUrlBuilder):
    def collections(self, offset: int = 0, limit: int = 100):
        return f'{self._endpoint}/launchpad/collections?offset={offset}&limit={limit}'


class MECollectionResponse:
    symbol = 'symbol'
    name = 'name'
    description = 'description'
    image = 'image'
    twitter = 'twitter'
    discord = 'discord'
    website = 'website'
    categories = 'categories'
    is_flagged = 'isFlagged'


class METokenResponse:
    mint_address = 'mintAddress'
    owner = 'owner'
    supply = 'supply'
    delegate = 'delegate'
    symbol = 'collection'
    name = 'name'
    update_authority = 'updateAuthority'
    primary_sale_happened = 'primarySaleHappened'
    seller_fee_bases_point = 'sellerFeeBasesPoint'
    image = 'image'
    animation_url = 'animationUrl'
    external_url = 'externalUrl'
    attributes = 'attributes'
    properties = 'properties'


class MEActivityResponse:
    signature = 'signature'
    type = 'type'
    source = 'source'
    token_mint = 'tokenMint'
    symbol = 'collection'
    slot = 'slot'
    block_time = 'blockTime'
    buyer = 'buyer'
    buyer_referral = 'buyerReferral'
    seller = 'seller'
    seller_referral = 'sellerReferral'
    price = 'price'


class MEOfferResponse:
    pda_address = 'pdaAddress'
    token_mint = 'tokenMint'
    auction_house = 'auctionHouse'
    buyer = 'buyer'
    price = 'price'
    token_size = 'tokenSize'
    expiry = 'expiry'


class MEEscrowBalanceResponse:
    buyer_escrow = 'buyerEscrow'
    balance = 'balance'


class MEListingResponse:
    pda_address = 'pdaAddress'
    auction_house = 'auctionHouse'
    token_address = 'tokenAddress'
    token_mint = 'tokenMint'
    seller = 'seller'
    token_size = 'tokenSize'
    price = 'price'


class MECollectionStatsResponse:
    symbol = 'symbol'
    floor_price = 'floorPrice'
    listed_count = 'listedCount'
    volume_all = 'volumeAll'


class MELaunchpadCollectionResponse:
    symbol = 'symbol'
    name = 'name'
    description = 'description'
    featured = 'featured'
    edition = 'edition'
    image = 'image'
    price = 'price'
    supply = 'size'
    launch_datetime = 'launchDatetime'



from datetime import datetime
from typing import Literal, TypedDict, List, Optional

MagicEdenAPIEnvironment = Literal['DEVNET ', 'MAINNET']


class METokenAttribute(TypedDict):
    trait_type: str
    value: str


class METokenFileProperty(TypedDict):
    uri: str
    type: str


class METokenCreatorProperty(TypedDict):
    address: str
    share: int


class METokenProperties(TypedDict):
    files: List[METokenFileProperty]
    category: str
    creators: List[METokenCreatorProperty]


class METoken(TypedDict):
    mint_address: str
    owner: str
    supply: int
    delegate: str
    symbol: str
    name: str
    update_authority: str
    primary_sale_happened: int
    seller_fee_basis_points: int
    image: str
    animation_url: Optional[str]
    external_url: str
    attributes: List[METokenAttribute]
    properties: METokenProperties


MEActivityType = Literal['BID', 'LIST', 'DELIST', 'BUY_NOW', 'CANCEL_BID']
MEActivitySource = Literal['magiceden', 'magiceden_v2']


class MEActivity(TypedDict):
    signature: str
    type: MEActivityType
    source: MEActivitySource
    token_mint: str
    symbol: str
    slot: int
    block_time: datetime
    buyer: str
    buyer_referral: str
    seller: str
    seller_referral: str
    price: float


class MEOffer(TypedDict):
    pda_address: str
    token_mint: str
    auction_house: str
    buyer: str
    price: float
    token_size: int
    expiry: int


class MEEscrowBalance(TypedDict):
    buyer_escrow: str
    balance: float


class MEListingItem(TypedDict):
    pda_address: str
    auction_house: str
    token_address: str
    token_mint: str
    seller: str
    token_size: int
    price: float


class MECollection(TypedDict):
    symbol: str
    name: str
    description: str
    image: str
    twitter: Optional[str]
    discord: Optional[str]
    website: Optional[str]
    categories: List[str]
    is_flagged: Optional[bool]


class MECollectionStats(TypedDict):
    symbol: str
    floor_price: float
    listed_count: int
    volume_all: float


class MELaunchpadCollection(TypedDict):
    symbol: str
    name: str
    description: str
    featured: Optional[bool]
    edition: str
    image: str
    price: float
    supply: int
    launch_datetime: datetime



import abc
import logging
from http import HTTPStatus
from json.decoder import JSONDecodeError
from typing import Optional, Dict, Union, List

import requests

logger = logging.getLogger(__name__)


class BaseApi(abc.ABC):
    def __init__(self, request_kwargs: dict = None):
        self._request_kwargs = request_kwargs
        self.request = None

    def _get_request(self, url: str, request_kwargs: dict = None) -> Union[Optional[Dict], Optional[List]]:
        req = requests.get(url, **{
            **self._request_kwargs,
            **(request_kwargs or {})
        })
        try:
            if req.status_code == HTTPStatus.OK:
                return req.json()
            logger.error(f'Request error. Status code: {req.status_code}')
        except JSONDecodeError as je:
            logger.error(f'Request error. Failed to parse json data. {str(je)}')
            return None
        except Exception as e:
            logger.error(f'Request error. {str(e)}')
        finally:
            self.request = req



from datetime import datetime


def parse_date(date: str):
    try:
        return datetime.fromisoformat(date.replace('T', ' ').replace('Z', '').split('.')[0])
    except (ValueError, TypeError, AttributeError):
        return None


def parse_timestamp(ts: int):
    try:
        return datetime.fromtimestamp(ts)
    except (ValueError, TypeError):
        return None



from typing import List, Dict

from magic_eden.api.base import MagicEdenOfficialApi
from magic_eden.api.utils.consts import MEAPILaunchpadUrlsBuilder
from magic_eden.api.utils.data import launchpad_collection_mapper
from magic_eden.api.utils.data import list_map
from magic_eden.api.utils.types import MELaunchpadCollection


class MagicEdenLaunchpadApi(MagicEdenOfficialApi):
    url_builder_class = MEAPILaunchpadUrlsBuilder
    url_builder: MEAPILaunchpadUrlsBuilder

    def collections_dirty(
            self,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[Dict]:
        url = self.url_builder.collections(offset, limit)
        return self._get_request(url, request_kwargs=request_kwargs)

    def collections(
            self,
            offset: int = 0,
            limit: int = 100,
            request_kwargs: dict = None
    ) -> List[MELaunchpadCollection]:
        dirty = self.collections_dirty(offset, limit, request_kwargs=request_kwargs)
        return list_map(launchpad_collection_mapper, dirty)



from setuptools import find_packages, setup

__version__ = "0.0.9"

setup(
    name='magic-eden-py',
    packages=find_packages(),
    version=__version__,
    description='Python wrap for api solana NFT marketplace MagicEden',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    project_urls={
        "Source Code": "https://github.com/ivannnnnnnnnn/magic-eden-py",
    },
    author='ivan.srshtn.crypto@gmail.com',
    license='MIT',
    install_requires=[
        'requests==2.27.0'
    ]
)
